#include "solver.h"

#include <queue>
using std::priority_queue;

class aStar : public Solver {
public:
	Plan plan;

	virtual bool solve(const State& s, const State& g, int p=0);
	aStar() : Solver("a*") {}
} aStarSolver;

class iterablePriorityQueue : public priority_queue<Plan>
{
public:
	iterablePriorityQueue() { c.reserve(1000); }

	bool contains(const State& s)
	{
		vector<Plan>::const_iterator it = c.begin();
		for (; it != c.end(); it++)
			if ( it->lastState.preds == s.preds ) break;

		return it != c.end();
	}

};

//It might not be fast, and the heuristic is potentially overstimating (so it might not ALWAYS
//return the best solution), but at least it solves hanoi without getting stuck
bool aStar::solve(const State& start, const State& gl, int p) {
	iterablePriorityQueue open;
	set<State> closed;

	Plan next;
	next.lastState = start;
	next.estimate = openGoals(next.lastState, gl);
	if (!next.estimate) return 1;
	open.push(next);

	while (!open.empty()) {
		plan = open.top();
		open.pop();
		//logger("Trying from "+plan.lastState.toString());

		vector<Operation>::iterator it=Operation::instances.begin();
		for (; it!=Operation::instances.end(); it++) {
			next = plan;

			next.lastState+=*it;

			//Skip, if operation isn't possible (not exactly "optimized")
			if (	next.lastState.preds == plan.lastState.preds ||
					open.contains(next.lastState) || 
					closed.find(next.lastState) != closed.end() )
				continue;
			
			//Add step to plan
			next+=&*it;
			int cost = openGoals(next.lastState, gl);
			//logger("  Adding "+it->toString());
			//"Are we there YET?"
			if (!cost) {Solver::solution=&(plan=next); return 1;}
			//Our also silly "heuristic" for the new state
			next.estimate = (int)next.steps.size() + cost;

			open.push(next);
			closed.insert(plan.lastState);
		}
	}
	return 0;	
}
