#ifndef ATOM_INCLUDED
#define ATOM_INCLUDED

#include "mapme.h"

#include <list>
using std::list;

//Halfhearted attempt of adding types for atoms
struct Type : public MapMe<Type> {
	static void wipe();
	const Type* parent;
	virtual void print() const {printf("%s", name.c_str()); if (parent) {printf("->"); parent->print();}}
	static Type* create(const char* n, const Type* t=0) {
		Type*& p = Map[n];
		if (p) error(string(n)+" already exists\n");
		return p ? p : p=new Type(n, t);
	}

protected:
	Type(const char* n, const Type* p=0) : MapMe<Type>(n), parent(p) {}
	friend struct MapMe<Type>;
} extern *anyType;

struct Atom : public MapMe<Atom> {	
	static void wipe();

	static Atom* create(const char* n, const Type* t) {
		Atom*& p = Map[n];
		if (p) error(string(n)+" already exists\n");
		return p ? p : p=new Atom(n,t);
	}
	//For later use.. someday
	const Type* type;

	virtual void print() const {type->print(); printf(":%s", name.c_str());}

protected:
	//Never call directly
	Atom(const char* n, const Type* t) : MapMe<Atom>(n), type(t) {}
};

struct Variable : Atom {
	//Makes memory management so much less painful
	static list<Variable*> variables;
	
	//Only call to add variables to predicates or operations, NOT for instances
	//variables for operation effects must be the ones passed as operation parameters
	//Why not just the constructor?
	//Because it prevents Variable::create from calling Atom::create
	static Variable* create(const char* n, const Type* t) {
		Variable* v = new Variable(n, t);
		variables.push_back(v);
		return v;
	}

	//Caution: only continue reading if pointers don't make your head go boom!
	//When instancing operations, the variables set this to the corresponding atom from the
	//passed vector of parameters. All variables used in effect predicates are only pointers
	//to the operation prototype parameters (ie. the actual variables, ignoring another
	//indirection). Ergo we just have to change the pointers in these predicates to the atoms we
	//set earlier in the actual variable objects. Pretty pictures would be helpful to explain this.
	Atom* value;

private:	
	Variable(const char* n, const Type* t) : Atom(n, t), value(0) {}
};

#endif
