#ifndef BIT_VECTOR_INCLUDED
#define BIT_VECTOR_INCLUDED

//Finally a place for pointless template magic
//To elaborate: sizeof(type) is known at compile time, so instead of calculating at runtime
//we abuse template trickery to make log2 a recursive look up at compile time
template<unsigned X> struct Log2 {enum {res = Log2<(X>>1)>::res + 1};};
template<> struct Log2<1> {enum {res=0};};
#define log2(x) Log2<x>::res/**/

//Use this, if compiler can't handle the above
/*static unsigned log2(unsigned x) {
	unsigned shift=0;
	while (x>1) {x>>=1; shift++;}
	return shift;
}/**/

const unsigned WORDL = sizeof(unsigned);
const unsigned BPWM1 = sizeof(unsigned)*8-1;
const unsigned SHIFT = log2(sizeof(unsigned)*8);

//Quick and dirty, as MS seems to be missing "minor" details like a word-iterator for vector<bool>
struct BitVec {
	vector<unsigned> words;
	BitVec() {}
	BitVec(size_t s) : words((s>>SHIFT)+1) {}

	inline bool get(size_t s) const {return 0!=(words[s>>SHIFT]&(1<<(s&BPWM1)));}
	inline void set(size_t s) {words[s>>SHIFT]|=(1<<(s&BPWM1));}
	inline void unset(size_t s) {words[s>>SHIFT]&=~(1<<(s&BPWM1));}
	inline void resize(size_t s) {words.resize((s>>SHIFT)+1);}
	inline void clear() {memset(&words[0],0,words.size()*WORDL);}
	
	inline operator bool() {
		for (size_t i=0; i<words.size(); i++) if (words[i]!=0) return 1;
		return 0;
	}
	inline BitVec operator&(const BitVec& b) const {
		size_t l=words.size()<b.words.size()?words.size():b.words.size();
		BitVec r(b);
		for (size_t i=0; i<l; i++) r.words[i]=words[i]&b.words[i];
		return r;
	}
	inline BitVec operator|(const BitVec& bv) const {
		const BitVec *va=this, *vb=&bv;
		if (va->words.size()>vb->words.size()) {vb=this; va=&bv;}
		size_t a=va->words.size(), b=vb->words.size();
		BitVec r; r.words.resize(b);
		for (size_t i=0; i<a; i++) r.words[i]=va->words[i]|vb->words[i];
		for (size_t i=a; i<b; i++) r.words[i]=vb->words[i];
		return r;
	}
	inline BitVec& operator&=(const BitVec& b) {
		size_t l=words.size()<b.words.size()?words.size():b.words.size();
		for (size_t i=0; i<l; i++) words[i]&=b.words[i];
		return *this;
	}
	inline BitVec& operator|=(const BitVec& b) {
		if (b.words.size()>words.size()) words.resize(b.words.size());
		for (size_t i=0; i<b.words.size(); i++) words[i]|=b.words[i];
		return *this;
	}
	inline bool operator==(const BitVec& bv) const {
		const BitVec *va=this, *vb=&bv;
		if (va->words.size()>vb->words.size()) {vb=this; va=&bv;}
		size_t a=va->words.size(), b=vb->words.size();
		for (size_t i=0; i<a; i++) if (va->words[i]!=vb->words[i]) return 0;
		for (size_t i=a; i<b; i++) if (vb->words[i]!=0) return 0;
		return 1;
	}
	inline bool operator<(const BitVec& b) const {
		if (words.size()<b.words.size()) return 1;
		size_t l=std::min(words.size(), b.words.size());
		for (size_t i=0; i<l; i++) if (words[i]!=b.words[i]) return  words[i]<b.words[i];
		return 0;
	}
};

#endif
