#include "constraints.h"

ConstInOf* Constraint::InOf(Variable* x, bool neg) {
	return x ? new ConstInOf(x,neg) : 0;
}

string ConstInOf::toString() const {
	string s = x->name;
	s+= negated ? "!=":"=";
	if (A.size()>1) s+="{";
	for (set<const Atom*>::const_iterator it=A.begin(); it!=A.end(); it++) {
		if (it!=A.begin()) s+="|";
		s+=(*it)->name.c_str();
	}
	for (set<const Type*>::const_iterator it=T.begin(); it!=T.end(); it++) {
		if (it!=T.begin()) s+="|";
		s+=(*it)->name.c_str();
	}
	return (A.size()+T.size()>1) ? s+="}" : s;
}

//Yes, ^ is bitwise and if any compiler uses DIFFERENT values for true it breaks
bool ConstInOf::check() const {
	if (T.find(x->value->type)!=T.end()) return !negated;
	return (A.find(x->value)!=A.end()) ? !negated : negated;
}

bool CondConst::check() const {
	bool apply = bOr ? 0 : 1;
	for (set<Constraint*>::const_iterator it=conditions.begin(); it!=conditions.end(); it++) {
		if (bOr && (*it)->check()) {apply=1; break;}
		if (!bOr && !(*it)->check()) {apply=0; break;}
	}
	//If constraints don't apply, pretend they are all matched
	if (!apply) return 1;

	//If any constraint fails we can return
	for (set<Constraint*>::const_iterator cit=consts.begin(); cit!=consts.end(); cit++)
		if (!(*cit)->check()) return 0;
	return 1;
}

string CondConst::toString() const {
	string s="[";
	for (set<Constraint*>::const_iterator it=conditions.begin(); it!=conditions.end(); it++) {
		if (it!=conditions.begin()) s+= bOr ? " v ":" ^ ";
		s+=(*it)->toString();
	}
	s+="]=>[";
	for (set<Constraint*>::const_iterator cit=consts.begin(); cit!=consts.end(); cit++) {
		if (cit!=consts.begin()) s+= " ^ ";
		s+=(*cit)->toString();
	}
	return s+="]";
}

CondConst::~CondConst() {
	for (set<Constraint*>::iterator it=conditions.begin(); it!=conditions.end(); it++) delete *it;
	for (set<Constraint*>::iterator cit=consts.begin(); cit!=consts.end(); cit++) delete *cit;
}
