#ifndef PLANGRAPH_INCLUDED
#define PLANGRAPH_INCLUDED

#include "operation.h"
#include "bitvector.h"

//The somewhat nondescript ptr member is the remnant of having this be a single template class
//Pointer to actual thing, exclusion bitvector for each layer and indices of connected ops/preds
struct PredNode {
	const Predicate* ptr;
	vector<BitVec> excl;
	vector<size_t> add;
	PredNode(const Predicate* p) : ptr(p) {}
};

struct OpNode {
	BitVec interference;
	const Operation* ptr;
	vector<BitVec> excl;
	vector<size_t> add, pre;
	OpNode(const Operation* o) : ptr(o) {}
};

//The only fancy part are the vectors keeping track of where a layer begins and ends
struct PlanGraph {
	list<Operation*> unusedOps;
	vector<OpNode> opNodes;
	vector<PredNode> predNodes;
	vector<size_t> opLayers, predLayers;
	bool changed;

	void buildLayer(int i);
	inline bool checkInterference(const OpNode& a, const OpNode& b);
	inline bool checkPredExclusion(const PredNode& a, const PredNode& b, const size_t& i);
	inline bool checkOpExclusion(const OpNode& a, const OpNode& b, const size_t& i);
	inline void addPredExclusions(int i);
	inline void addOpExclusions(int i);	
	inline bool addOpIfPossible(const Operation& o, int i);

	inline void clear() {
		opNodes.clear(); predNodes.clear();
		opLayers.clear(); predLayers.clear();
		unusedOps.clear(); changed=1;
	}
	
	inline size_t newOp(const Operation* o) {opNodes.push_back(OpNode(o)); return opNodes.size()-1;}
	inline size_t newPred(const Predicate* p) {predNodes.push_back(PredNode(p)); return predNodes.size()-1;}
	
	inline void finishLayer() {
		//Just to make the compiler stfu
		size_t a=opNodes.size();
		opLayers.push_back(a);
		a = predNodes.size();
		if (!predLayers.empty() && a!=predLayers.back()) changed=1;
		predLayers.push_back(a);
	}
};

#endif
