#include "solver.h"
#include "graphplan.h"
#include <stack>
using std::stack;

Graphplan graphSolver;
static int r=0, h=0;

bool Graphplan::searchRec(int lvl, vector<size_t>& goals) {
	r++;
	if (lvl<=0) return 1;
	int i = (int)graph.predLayers.size()-2;
	if (lvl<i) i=lvl;

	vector<size_t> newGoals;
	list<size_t> candidates;
	stack<BitVec> goalVec; goalVec.push(BitVec(graph.predLayers[i+1]));
	stack<BitVec> exclVec; exclVec.push(BitVec(graph.opLayers[i+1]));
	vector<vector<size_t> > goalDigits(goals.size());
	vector<vector<size_t>::iterator> digIter(goals.size());
	int digit=0;

	//Clockwork time
	//First each "digit" is collecting its possible values
	for (vector<size_t>::const_iterator g=goals.begin(); g!=goals.end(); g++, digit++) {
		for (vector<size_t>::const_iterator o=graph.predNodes[*g].add.begin(); o!=graph.predNodes[*g].add.end(); o++) {
			//Important: we might have edges to operations that didn't exist so early in the graph,
			//so we need to check for this (ex. can't have a no op when a predicate first appeared)
			if (*o<graph.opLayers[i+1]) goalDigits[digit].push_back(*o);
		}
		digIter[digit]=goalDigits[digit].begin();
	}
	digit=0;

	while(digIter[0]!=goalDigits[0].end()) {
		//Let's build our selection of actions for today
		//Check each position against all previously selected positions for exclusion
		//If it doesn't cause trouble or the goal is already covered, use it
		//This loop seems to use most of the time for searching
		//Changed: now using stacks instead of rebuilding the whole thing for each "digit" ->.2-5x faster now
		
		for (digit; digit<(int)goals.size(); digit++) {
			goalVec.push(goalVec.top());
			exclVec.push(exclVec.top());
			if (exclVec.top().get(*digIter[digit])) {
				digit++;
				goto SkipCombination;
			}
			if (goalVec.top().get(goals[digit])) continue;
						
			const OpNode& op = graph.opNodes[*digIter[digit]];
			exclVec.top()|=op.excl[i];
			candidates.push_back(*digIter[digit]);
			for (vector<size_t>::const_iterator it=op.add.begin(); it!=op.add.end(); it++) 
				goalVec.top().set(*it);
		}

		//Fine, all set, let's try how far it gets us
		//Add all the preconditions to the list of new goals
		goalVec.top().clear();
		newGoals.clear();
		
		for (list<size_t>::iterator l=candidates.begin(); l!=candidates.end(); l++) {
			for (vector<size_t>::iterator pre=graph.opNodes[*l].pre.begin(); pre!=graph.opNodes[*l].pre.end(); pre++) {
				if (!goalVec.top().get(*pre)) {
					newGoals.push_back(*pre);
					goalVec.top().set(*pre);
				}
			}
		}
		//Accidental: sorting goals by index (ie. age) helps and let's us work on the "new" goals first
		std::sort(newGoals.begin(), newGoals.end());
		for (HashTable::iterator it=hashTable.begin(); it!=hashTable.end(); it++) {
			if (it->second<lvl) continue;
			if ((goalVec.top()&it->first)==it->first) {
				h++;
				goto SkipCombination;
			}
		}

		//Cross fingers, hope to find a solution, if so, add selected actions to plan and return success
		if (searchRec(lvl-1, newGoals)) {
			plan.steps.resize(plan.steps.size()+1);
			for (list<size_t>::iterator l=candidates.begin(); l!=candidates.end(); l++) {
				if (graph.opNodes[*l].ptr)
					plan.steps.back().push_back(const_cast<Operation*>(graph.opNodes[*l].ptr));
			}
			return 1;
		}
		hashTable[goalVec.top()]=lvl;
		//else we set the iterators to the next combination
SkipCombination:
		for (--digit; digit>=0; digit--) {
			exclVec.pop(); goalVec.pop(); candidates.remove(*digIter[digit]);
			if (++digIter[digit]!=goalDigits[digit].end()) break;
			if (digit>0) digIter[digit]=goalDigits[digit].begin();
		}
	}
	return 0;
}

inline bool Graphplan::findPath(int i) {
	int level = (int)graph.predLayers.size()-2;
	if (i<level) level=i;
	vector<size_t> goals;
	//Check if all goals are part of the last predicate layer and non exclusive
	for (set<Predicate*>::iterator g=goal.preds.begin(); g!=goal.preds.end(); g++) {
		size_t p = 0;
		for (; p<graph.predLayers[level+1]; p++) {
			if (graph.predNodes[p].ptr==*g) {
				//Found but excluded by previous goal?
				for (size_t gi=0; gi<goals.size(); gi++) 
					if (graph.predNodes[goals[gi]].excl[level].get(p)) return 0;
				goals.push_back(p);
				break;
			}
		}
		//Goal pred not found
		if (p==graph.predLayers[level+1]) return 0;
	}
	return searchRec(i, goals);
}

bool Graphplan::solve(const State& s, const State& g, int) {
	hashTable.clear();
	graph.clear();
	graph.opNodes.reserve(Operation::instances.size()+Predicate::instances.size());
	graph.predNodes.reserve(Predicate::instances.size());
	
	//First copy all initial predicates into layer 0 predicate nodes
	for (size_t i=0; i<Operation::instances.size(); i++) graph.unusedOps.push_back(&Operation::instances[i]);
	graph.finishLayer();
	for (set<Predicate*>::const_iterator pit=s.preds.begin(); pit!=s.preds.end(); pit++) {
		size_t p = graph.newPred(*pit);
		graph.predNodes[p].excl.resize(1);
		graph.predNodes[p].excl[0].resize(s.preds.size());
	}
	graph.finishLayer();

	int i=0;
	r=0, h=0;
	uint64 tl=0, ts=0;
	size_t prevHashEntries=0;
	bool result=0, changes=0;

	while (++i) {
		uint64 tmp=ticks();
		graph.buildLayer(i);
		tl+=ticks()-tmp;
		tmp=ticks();
		result=findPath(i);
		ts+=ticks()-tmp;
		if (result) {
			Solver::solution=&plan;
			break;
		}
	
		//Check if anything has changed at all
		if (!graph.changed) {
			if (hashTable.size()==prevHashEntries) break;
			else prevHashEntries=hashTable.size();
		}
	}
	char tmp[256];
	_snprintf(tmp, 255, "Graph (d%d): %.2fms, Search: %.2fms (%d recs, hash entries/hits %u/%d)\n",
		i, (float)tl/2000000.0f,  (float)ts/2000000.0f, r, hashTable.size(), h);
	logger(tmp);
	 
	return result;
}



//Was required for fixing plans created by the cheating version.. which became more and more pointless
//the more I optimized the regular search

/*
inline bool deletes(const Operation* a, const Operation* b) {
	for (set<Predicate*>::const_iterator del = a->del.preds.begin(); del!=a->del.preds.end(); del++) {
		if (b->add.preds.find(*del)!=b->add.preds.end()) return 1;
	}
	return 0;
}
inline bool prevents(const Operation* a, const Operation* b) {
	for (set<Predicate*>::const_iterator del = a->del.preds.begin(); del!=a->del.preds.end(); del++) {
		if (b->pre.preds.find(*del)!=b->pre.preds.end()) return 1;
	}
	return 0;
}

bool refinePlan() {
	ParPlan newPlan;
	plan.lastState=init;

	for (size_t i=0; i<plan.steps.size(); i++) {
		if (plan.steps[i].empty()) continue;
		newPlan.steps.push_back(list<Operation*>());		
		if (plan.steps.size()<=i+1) plan.steps.push_back(list<Operation*>());

		//Everything that can't be done yet must wait
		for (list<Operation*>::iterator it = plan.steps[i].begin(); it!=plan.steps[i].end();) {
			list<Operation*>::iterator tmp=it++;
			for (set<Predicate*>::const_iterator pre=(*tmp)->pre.preds.begin(); pre!=(*tmp)->pre.preds.end(); pre++) {
				if (!plan.lastState.elem(*pre)) {
					plan.steps[i+1].push_back(*tmp);
					plan.steps[i].erase(tmp);
					break;
				}
			}
		}

		list<Operation*> toMove;
		//Anything preventing another action or having its effects deleted must be done later
		for (list<Operation*>::iterator it1 = plan.steps[i].begin(); it1!=plan.steps[i].end(); it1++) {
			for (list<Operation*>::iterator it2 = plan.steps[i].begin(); it2!=plan.steps[i].end(); it2++) {
				if (it1==it2) continue;
				if (prevents(*it1, *it2) || deletes(*it2, *it1)) toMove.push_back(*it1);
			}
		}
		//Remove everything that had to moved
		for (list<Operation*>::iterator m=toMove.begin(); m!=toMove.end(); m++) {
			plan.steps[i].remove(*m);
			plan.steps[i+1].push_back(*m);
		}
		//The rest is added to the new plan and applied to the current state
		for (list<Operation*>::iterator it = plan.steps[i].begin(); it!=plan.steps[i].end(); it++) {
			newPlan.steps.back().push_back(*it);
			plan.lastState+=**it;
		}
		//Nothing we could do this time, so we are stuck and give up (which happens whenever 2 or more
		//operations are caught in some circular interference that we can't break up)
		if (newPlan.steps.back().empty()) {plan.clear(); return 0;}
	}
	plan=newPlan;
	return 1;
}/**/
