#include "solver.h"

class insel : public Solver {
public:
	Plan plan;
	virtual bool solve(const State& s, const State& g, int p=0);
	insel() : Solver("insel") {}
	inline Operation* getA(const State&s, const State& g);
	bool inselRec(State& st, const State& gl, int d);
} inselSolver;

//Iterate over all operation instances and return the first where at least one add-effect
//is part of the goal state and not already part of the initial state
inline Operation* insel::getA(const State&s, const State& g) {
	vector<Operation>::iterator it=Operation::instances.begin();
	for (; it!=Operation::instances.end(); it++) {
		set<Predicate*>::iterator pit=it->add.preds.begin();
		for (; pit!=it->add.preds.end(); pit++) {
			if (g.elem(*pit) && !s.elem(*pit)) return &*it;
		}
	}
	return 0;
}

//Original version, extremely prone to get stuck in infinite loops
bool insel::inselRec(State& st, const State& gl, int d) {
	if (d>100) return 0;
	while(!goalReached(st, gl)) {
		logger("\nTrying to achieve: "+gl.toString());
		Operation* a=getA(st, gl);
		if (!a) return 0;
		logger("Picked: " + a->toString());
		if (!inselRec(st, a->pre, d+1)) return 0;
		st+=*a;
		plan+=a;
		logger("New State: "+init.toString());
	}
	Solver::solution=&plan;
	return 1;
}

bool insel::solve(const State& st, const State& gl, int p) {
	State start = st;
	return inselRec(start, gl, 0);
}
