#include "solver.h"

struct OpWithCount {
	Operation* op;
	int count;
};

class inselOL : public Solver {
public:
	Plan plan;
	virtual bool solve(const State& s, const State& g, int p=0);
	inselOL() : Solver("inselol") {}
	list<OpWithCount> ops;
	inline list<OpWithCount*> buildA(const State&s, const State& g);
	bool inselOpRec(State& st, const State& gl, int lim, int d);
} inselOLSolver;

inline list<OpWithCount*> inselOL::buildA(const State&s, const State& g) {
	list<OpWithCount*> A;
	for (list<OpWithCount>::iterator it=ops.begin(); it!=ops.end(); it++) {
		set<Predicate*>::iterator pit=it->op->add.preds.begin();
		for (; pit!=it->op->add.preds.end(); pit++) 
			if (g.elem(*pit) && !s.elem(*pit)) A.push_back(&*it);
	}
	return A;
}

//Next try: we count how often any operation was used so far and check that against a limit
//If it's used too often already, we skip it
bool inselOL::inselOpRec(State& st, const State& gl, int lim, int d) {
	if (d>300) return 0; //Keep stack from blowing up
	while(!goalReached(st, gl)) {
		logger("\nTrying to achieve: "+gl.toString());
		//Instead of using just any random a we use them all until it works out (or not)
		list<OpWithCount*> A = buildA(st, gl);
		list<OpWithCount*>::iterator a=A.begin();
		for (; a!=A.end(); a++) {
			if ((*a)->count++>lim) continue;
			logger("Picked: " + (*a)->op->toString());
			if(inselOpRec(st, (*a)->op->pre, lim, d+1)) break;
			else (*a)->count--;
		}
		if (a==A.end()) return 0;
		st+=*(*a)->op;
		plan+=(*a)->op;
		logger("New State: "+st.toString());
	}
	Solver::solution=&plan;
	return 1;
}

bool inselOL::solve(const State& s, const State& g, int ol) {
	vector<Operation>::iterator it=Operation::instances.begin();
	for (; it!=Operation::instances.end(); it++) {
		OpWithCount op={&*it, 0};
		ops.push_back(op);
	}
	State start = s;
	return inselOpRec(start, g, ol, 0);
}
