#include "solver.h"

class inselPL : public Solver {
public:
	Plan plan;
	virtual bool solve(const State& s, const State& g, int p=0);
	inselPL() : Solver("inselpl") {}
	bool inselPLRec(State& st, const State& gl, int lim, int d);
	inline list<Operation*> buildA(const State&s, const State& g);
} inselPLSolver;

//Almost like getA in insel.cpp, but collecting all possible operations in a list
inline list<Operation*> inselPL::buildA(const State&s, const State& g) {
	list<Operation*> A;
	vector<Operation>::iterator it=Operation::instances.begin();
	for (; it!=Operation::instances.end(); it++) {
		set<Predicate*>::iterator pit=it->add.preds.begin();
		for (; pit!=it->add.preds.end(); pit++) 
			if (g.elem(*pit) && !s.elem(*pit)) A.push_back(&*it);
	}
	return A;
}

//Should prevent most loops
//Instead of limiting only the recursion depth, we limit the plan length
//Though technically not an infinite loop, it still takes long enough to feel like one
bool inselPL::inselPLRec(State& st, const State& gl, int lim, int d) {
	while(!goalReached(st, gl)) {
		if (++d>lim) return 0;
		logger("\nTrying to achieve: "+gl.toString());
		//Instead of using just any random a we use them all until it works out (or not)
		list<Operation*> A = buildA(st, gl);
		list<Operation*>::iterator a=A.begin();
		Plan backupPL = plan;
		State backupST = st;
		for (; a!=A.end(); a++) {
			logger("Picked: " + (*a)->toString());
			if (inselPLRec(st, (*a)->pre, lim, d+1)) break;
			else if (plan.steps.size()!=backupPL.steps.size()) {
				plan = backupPL;
				st = backupST;
			}
		}
		if (a==A.end()) return 0;
		st+=**a;
		plan+=*a;
		logger("New State: "+st.toString());
	}
	Solver::solution=&plan;
	return 1;
}

bool inselPL::solve(const State& st, const State& gl, int lim) {
	State start = st;
	return inselPLRec(start, gl, lim, 0);
}
