#include "solver.h"

#include <stdlib.h>

class inselRand : public Solver {
public:
	Plan plan;
	virtual bool solve(const State& s, const State& g, int p=0);
	inselRand() : Solver("inselrand") {}
	inline Operation* getRandA(const State&s, const State& g);
	bool inselRandRec(State& st, const State& gl, int d);
} inselRand;

inline Operation* inselRand::getRandA(const State&s, const State& g) {
	vector<Operation*> A;
	vector<Operation>::iterator it=Operation::instances.begin();
	for (; it!=Operation::instances.end(); it++) {
		set<Predicate*>::iterator pit=it->add.preds.begin();
		for (; pit!=it->add.preds.end(); pit++) 
			if (g.elem(*pit) && !s.elem(*pit)) A.push_back(&*it);
	}
	return A.size() ? A[rand()%A.size()] : 0;
}

//This time we try picking a random operation, with a fix depth limit to prevent
//the stack from blowing up
bool inselRand::inselRandRec(State& st, const State& gl, int d) {
	if (d>300) return 0;
	while(!goalReached(st, gl)) {
		logger("\nTrying to achieve: "+gl.toString());
		Operation* a=getRandA(st, gl);
		if (!a) return 0;
		logger("Picked: " + a->toString());
		if (!inselRandRec(st, a->pre, d+1)) return 0;
		st+=*a;
		plan+=a;
		logger("New State: "+init.toString());
	}
	Solver::solution=&plan;
	return 1;
}

bool inselRand::solve(const State& st, const State& gl, int p) {
	State start = st;
	return inselRandRec(start, gl, 0);
}
