#ifndef GUI_DMWINDOW_INCLUDED
#define GUI_DMWINDOW_INCLUDED

#include "window.h"
#include "../atom.h"

struct DMWindow : Window {
	float posOffset[2];

	DMWindow() 
		: Window(&guiRoot,"Domain Information",2.65f,.1f,97.25f, 99.8f) {
			posOffset[0]=0.0f; posOffset[1]=0.0f;
			setBgColor(128,180,255);
	}

	virtual void mouseDrag(float dx, float dy) {
		posOffset[1]+=dy;
		if (posOffset[1]>0) posOffset[1]=0;
	}
	virtual void wheel(float dz) {mouseDrag(0, dz*5);}

	virtual void draw() {
	if (!isOpen && !isOpening) return;
	Window::draw();
	glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(0, posOffset[1], 0);

	glColor3f(0,0,0);
	Font::Print("Types", 1.5f, 1.5f, 2);
	string out;
	for (map<string, Type*>::iterator it=Type::Map.begin(); it!=Type::Map.end(); it++) {
		if (it!=Type::Map.begin()) out+=", ";
		out+=it->second->name;
		if (it->second->parent) out+="->"+it->second->parent->name;
	}
	Font::PrintArea(out, 15.0f, 1.5f, 30.0f, 20.0f, 2.0f);

	out.clear();
	Font::Print("Atoms", 1.5f, 21.5f, 2);
	for (map<string, Atom*>::iterator it=Atom::Map.begin(); it!=Atom::Map.end(); it++) {
		if (it!=Atom::Map.begin()) out+=", ";
		out+=it->second->name + "["+it->second->type->name+"]";
	}
	Font::PrintArea(out, 15.0f, 21.5f, 30.0f, 20.0f, 2.0f);

	out.clear();
	Font::Print("Predicates", 1.5f, 41.5f, 2);
	for (map<string, Predicate*>::iterator it=Predicate::Map.begin(); it!=Predicate::Map.end(); it++) {
		if (it!=Predicate::Map.begin()) out+=", ";
		out+=it->second->toString();
	}
	Font::PrintArea(out, 15.0f, 41.5f, 30.0f, 20.0f, 2.0f);

	Font::Print("Initial\nState", 1.5f, 61.5f, 2);
	Font::PrintArea(init.toString(), 15.0f, 61.5f, 30.0f, 15.0f, 2.0f);

	Font::Print("Goal\nState", 1.5f, 76.5f, 2);
	Font::PrintArea(goal.toString(), 15.0f, 76.5f, 30.0f, 15.0f, 2.0f);
	
	out.clear();
	Font::Print("Operations", 50.5f, 1.5f, 2);
	for (map<string, Operation*>::iterator it=Operation::Map.begin(); it!=Operation::Map.end(); it++) {
		string head =it->second->toString();
		char tmp[32]; _snprintf(tmp,31," - (%d)\n",it->second->countInstances());
		out+=string("\n")+head.substr(0, head.size()-1).c_str()+tmp;
		out+="   pre:\t" + it->second->pre.toString();
		out+="   add:\t" + it->second->add.toString();
		out+="   del:\t" + it->second->del.toString();
		out+="   const:\t";
		for (set<Constraint*>::const_iterator itc=it->second->constraints.begin(); itc!=it->second->constraints.end(); itc++) {
			if (itc!=it->second->constraints.begin()) out+=", ";
			out+=(*itc)->toString();
		}
		out+="\n";
	}
	Font::PrintArea(out, 50.5f, 8.5f, 40.0f, .0f, 2.0f);
	glDisable(GL_TEXTURE_2D);

	glBegin(GL_LINES);
		glVertex2f(2,20);
		glVertex2f(45,20);
		glVertex2f(2,40);
		glVertex2f(45,40);
		glVertex2f(2,60);
		glVertex2f(45,60);
		glVertex2f(2,75);
		glVertex2f(45,75);
		glVertex2f(47,2);
		glVertex2f(47,94);
	glEnd();
	
	glPopMatrix();
	
	}
};

#endif