#include "glfw.h"
#include "glfont.h"

bool Font::Initialized=0;
unsigned Font::Tex=0, Font::TexWidth=0, Font::TexHeight=0, Font::TexStart=0, Font::TexEnd=0;
Font::CHAR* Font::Char=0;

void Font::Init(char* filename) {
	if (Char) delete[] Char;

	FILE *Input;
	Input = fopen(filename, "rb");
	if (!Input) return;

	fread(&Tex, sizeof(int), 1, Input);
	fread(&TexWidth, sizeof(int), 1, Input);
	fread(&TexHeight, sizeof(int), 1, Input);
	fread(&TexStart, sizeof(int), 1, Input);
	fread(&TexEnd, sizeof(int), 1, Input);
	fread(&Char, sizeof(CHAR*), 1, Input);

	int Num = TexEnd - TexStart + 1;
	Char = new CHAR[Num];
	fread(Char, sizeof(CHAR), Num, Input);

	Num = TexWidth * TexHeight * 2;
	char *TexBytes = new char[Num];
	fread(TexBytes, sizeof(char), Num, Input);

	unsigned ID;
	glGenTextures(1, &ID);
	Tex=ID;
	glBindTexture(GL_TEXTURE_2D, Tex);
	glTexImage2D(GL_TEXTURE_2D, 0, 2, TexWidth, TexHeight, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, (void *)TexBytes);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_CLAMP);
	
	delete []TexBytes;
	fclose(Input);

	Initialized=1;
}

void Font::Cleanup() {
	if (Char) delete[] Char;
	glDeleteTextures(1, &Tex);
}

float Font::width(std::string String, float sz) {
	if (!Initialized) return -1.0f;
	float x=0.0f, y=0.0f;
	float wFac=sz/Char->height;
	int Length = (int)String.length();
	float wdth=0;
	for (int i = 0; i < Length; i++) {
		if (String[i]=='\n') {x=0; y+=sz; continue;}
		if (String[i]=='\t') {x=sz*2 + (int)(x/sz*2)*sz*2; continue;}
		x+=Char[(int)String[i] - TexStart].width*wFac;
		if (x>wdth) wdth=x;
	}
	return wdth;
}

float Font::Print(std::string String, float x, float y, float sz, bool right) {
	if (!Initialized) return -1.0f;
	float wFac=sz/Char->height;
	float Tab=sz*2;
	int Length = (int)String.length();
	if (right) {
		float linelength=0, longest=0;
		for (int i = 0; i < Length; i++) {
			if (String[i]=='\n') {linelength=0; continue;}
			linelength+=Char[(int)String[i] - TexStart].width*wFac;
			if (linelength>longest) longest=linelength;
		}
		x-=longest;
	}
	float X=x;
	float longest=x;

	glBindTexture(GL_TEXTURE_2D, Tex);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glBegin(GL_QUADS);
		float DX, DY=sz;
		for (int i = 0; i < Length; i++) {
			if (String[i]=='\n') {x=X; y+=DY; continue;}
			if (String[i]=='\t') {
				x=Tab+(int)(x/Tab)*Tab; continue;}
			
			CHAR *Ch = &Char[(int)String[i] - TexStart];
			DX=Ch->width*wFac;

			glTexCoord2f(Ch->tx1, Ch->ty1);
			glVertex3f(x, y, 0);
			glTexCoord2f(Ch->tx1, Ch->ty2);
			glVertex3f(x, y + DY, 0);
			glTexCoord2f(Ch->tx2, Ch->ty2);
			glVertex3f(x + DX, y + DY, 0);
			glTexCoord2f(Ch->tx2, Ch->ty1);
			glVertex3f(x + DX, y, 0);
			x+=DX;
			if (x > longest) longest=x;
		}
	glEnd();
	return longest-X;
}

void Font::PrintArea(std::string String, float x, float y, float w, float h, float sz) {
	if (!Initialized) return;
	float X=x;
	float wFac=sz/Char->height;
	int Length = (int)String.length();
	glBindTexture(GL_TEXTURE_2D, Tex);

	int lastword=0, i=0;
	float width=0;
	float Tab=sz*2;
	while (i < Length) {
		if (String[i]=='\n') {width=0; lastword=i;}		
		else {
			if (String[i]==' ') lastword=i;
			else if (String[i]=='\t') {lastword=i; width=Tab+(int)(width/Tab)*Tab;}
			else {
				CHAR *Ch = &Char[(int)String[i] - TexStart];
				width+=Ch->width*wFac;
			}
			if (width > w && lastword && String[lastword]!='\n') {
				i=lastword;
				String[i]='\n';
				width=0;
			}
		}
		++i;
	}

	float height=sz;
	glBegin(GL_QUADS);
		float DX, DY=sz;
		for (int j = 0; j < Length; j++) {
			if (String[j]=='\n') {x=X; y+=DY; height+=sz; continue;}
			if (String[j]=='\t') {x=Tab+(int)(x/Tab)*Tab; continue;}
			if (height > h && h>0) break;
			CHAR *Ch = &Char[(int)String[j] - TexStart];

			DX=Ch->width*wFac;
			glTexCoord2f(Ch->tx1, Ch->ty1);
			glVertex3f(x, y, 0);
			glTexCoord2f(Ch->tx1, Ch->ty2);
			glVertex3f(x, y + DY, 0);
			glTexCoord2f(Ch->tx2, Ch->ty2);
			glVertex3f(x + DX, y + DY, 0);
			glTexCoord2f(Ch->tx2, Ch->ty1);
			glVertex3f(x + DX, y, 0);
			x += DX;
		}
	glEnd();
}
