#ifndef GUI_BASE_INCLUDED
#define GUI_BASE_INCLUDED

#include "ogl.h"

#include <vector>
#include <string>
using std::vector;
using std::string;

struct GuiElement;
struct Window;
extern GuiElement guiRoot;

struct GuiElement {
	static float aniSpeed;
	static int WinX, WinY, mousePos[3];
	static GuiElement* mouseOver, *popUp;
	static Window* openWindow;
	
	vector<GuiElement*> childs;
	float x,y,width,height;
	unsigned char bgColor[3];

	static void makePopUp(GuiElement* p) {
		delete popUp; 
		popUp=p;
		if (guiRoot.childs.back()==p) guiRoot.childs.pop_back();
	}

	virtual void setBgColor(unsigned char r, unsigned char g, unsigned b) {
		bgColor[0]=r; bgColor[1]=g; bgColor[2]=b;
	}

	virtual GuiElement* pickElement(float mx, float my) {
		if (mx<x || mx>x+width || my<y || my>y+height) return 0;
		GuiElement* picked=0;
		for (int i=(int)childs.size()-1; i>=0 && !picked ; --i) 
			picked=childs[i]->pickElement(mx-x, my-y);
		return picked ? picked : this;
	}

	virtual void setPos(float ix, float iy) {x=ix; y=iy;}
	virtual void setSize(float iw, float ih) {width=iw; height=ih;}

	virtual void update(float dtime) {
		for (size_t i=0; i<childs.size(); i++) childs[i]->update(dtime);
	}

	virtual void mouseEnter() {}
	virtual void mouseLeave() {}

	virtual void render() {
		glPushMatrix();
		glTranslatef(x,y,0);
		draw();
		for (size_t i=0; i<childs.size(); i++) childs[i]->render();
		glPopMatrix();
	}
	virtual void draw() {
		glColor3ubv(bgColor);
		glBegin(GL_QUADS);
		glVertex2f(0,  0);
		glVertex2f(0, height);
		glVertex2f(width, height);
		glVertex2f(width,  0);
		glEnd();
	}
	virtual void click() {}
	virtual void wheel(float dz) {}
	virtual void mouseMove(float mx, float my) {}
	virtual void mouseDrag(float dx, float dy) {}

	GuiElement(GuiElement* p=&guiRoot, float ix=0, float iy=0, float w=10, float h=10)
		: x(ix), y(iy), width(w), height(h) {
			if (p) p->childs.push_back(this);
			memset(bgColor, 0, 3);
	}
};

#endif