#ifndef GUI_IFWINDOW_INCLUDED
#define GUI_IFWINDOW_INCLUDED

#include "window.h"
#include "label.h"
#include "../graphplan.h"
#include <cmath>

struct IFWindow : Window {
	PlanGraph& g;
	float scaleFac;
	float posOffset[2];
	float tleft, ttop;
	float cellwidth, cellheight;

	IFWindow() 
		: Window(&guiRoot,"Interference Table",2.65f,.1f, 97.25f, 99.8f), g(graphSolver.graph), scaleFac(1.0f),
		cellheight(3), cellwidth(9.0f/4.0f) {
			posOffset[0]=-50.0f; posOffset[1]=-50.0f;
			setBgColor(128,180,255);
	}

	virtual void mouseLeave() {GuiElement::makePopUp(0);}

	virtual void mouseMove(float mx, float my) {
		float fi=((mx-50)/scaleFac - (x+tleft+posOffset[0])) / cellwidth;
		float fk=((my-50)/scaleFac - (y+ttop+posOffset[1])) / cellheight;
		if (fi<0 || fk<0) {
			GuiElement::makePopUp(0);
			return;
		}
		size_t i=(size_t)fi, k=(size_t)fk;

		if (i<g.opNodes.size() && k<g.opNodes.size()) {
			string a = g.opNodes[i].ptr ? g.opNodes[i].ptr->toString() :
			"nop("+g.predNodes[g.opNodes[i].pre.front()].ptr->toString()+")\n";
			a += g.opNodes[k].ptr ? g.opNodes[k].ptr->toString() :
			"nop("+g.predNodes[g.opNodes[k].pre.front()].ptr->toString()+")\n";
			GuiElement::makePopUp(new Label(0, a, 0,0,0,4.25f));
		}
		else GuiElement::makePopUp(0);
	}

	virtual void wheel(float dz) {		
		float scaleOld = scaleFac;
		scaleFac*=1+dz*.1f;
		posOffset[0]+=(mousePos[0]*100.0f - 50*WinX)*(scaleOld - scaleFac) / (WinX*scaleOld*scaleFac);
		posOffset[1]+=(mousePos[1]*100.0f - 50*WinY)*(scaleOld - scaleFac) / (WinY*scaleOld*scaleFac);
	}
	virtual void mouseDrag(float dx, float dy) {
		posOffset[0]+=dx/scaleFac;
		posOffset[1]+=dy/scaleFac;
	}

	virtual void draw() {
	if (!isOpen && !isOpening) return;
	Window::draw();
	if (g.predLayers.empty()) {close(); return;}

	const float angle=60.0f, angFacH=cosf(angle*3.1415f/180.0f), angFacW=sinf(angle*3.1415f/180.0f);
	const float fntSz=2.0f;		
	float widest=0, w=0;

	for (size_t outer=0; outer<g.opNodes.size(); outer++) {
		w = g.opNodes[outer].ptr ? Font::width(g.opNodes[outer].ptr->toString(), fntSz) :
			Font::width("nop("+g.predNodes[g.opNodes[outer].pre.front()].ptr->toString()+")", fntSz);
		if (w>widest) widest=w;
	}

	tleft=widest+2;
	ttop=widest*angFacW + 2;
	const float width=tleft+g.opNodes.size()*cellwidth;
	const float height=ttop+g.opNodes.size()*cellheight;
	float left=tleft, top=0;

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	float sz=50.0f/scaleFac;
	glOrtho(-sz, sz, sz, -sz, -1,1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(posOffset[0], posOffset[1], 0);

	glBegin(GL_QUADS);
	glColor3f(1,.3f,.3f);
	for (size_t outer=0; outer<g.opNodes.size(); outer++, left+=cellwidth) {
		top=ttop;
		for (size_t inner=0; inner<g.opNodes.size(); inner++, top+=cellheight) {
			if (g.opNodes[outer].interference.get(inner)) {
				glVertex2f(left+cellwidth/4, top+cellheight/4);
				glVertex2f(left+cellwidth/4, top+3*cellheight/4);
				glVertex2f(left+3*cellwidth/4, top+3*cellheight/4);
				glVertex2f(left+3*cellwidth/4, top+cellheight/4);
			}
		}
	}
	glEnd();
	glColor3f(0,0,0);
	glBegin(GL_LINES); 
	for (left=tleft; left<=width; left+=cellwidth) {
		glVertex2f(left, ttop);
		glVertex2f(left, height);	
	}
	for (float top=ttop; top<=height; top+=cellheight) {
		glVertex2f(tleft, top);
		glVertex2f(width, top);
	 }
	glEnd();
	glEnable(GL_TEXTURE_2D);

	left=tleft-cellwidth/2.0f, top=ttop+(cellheight-fntSz)/2.0f;
	for (size_t outer=0; outer<g.opNodes.size(); outer++, top+=cellheight) {
		if (g.opNodes[outer].ptr) Font::Print(g.opNodes[outer].ptr->toString(), left, top, fntSz, 1);
		else Font::Print("nop("+g.predNodes[g.opNodes[outer].pre.front()].ptr->toString()+")", left, top, fntSz, 1);
	}

	left=tleft;
	for (size_t outer=0; outer<g.opNodes.size(); outer++, left+=cellwidth) {
		glPushMatrix();
		glTranslatef(left+angFacH*(cellwidth-fntSz)/2.0f,ttop-cellheight/2.0f,0); 
		glRotatef(angle,0,0,-1);
		if (g.opNodes[outer].ptr) Font::Print(g.opNodes[outer].ptr->toString(), 0, 0, fntSz);
		else Font::Print("nop("+g.predNodes[g.opNodes[outer].pre.front()].ptr->toString()+")", 0, 0, fntSz);
		glPopMatrix();
	}
	glDisable(GL_TEXTURE_2D);

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	}
};

#endif