#ifndef GUI_OCBUTTON_INCLUDED
#define GUI_OCBUTTON_INCLUDED

#include "window.h"
#include "button.h"
#include "../operation.h"

struct OCButton : Button {
	Window& win;
	OCButton(Window& w, string s, float x, float y, float wi, float h) 
		: Button(0, s, x,y,wi,h), win(w) {}
	virtual void click() {
		win.minimize();
	}
};

struct ERRButton : Button {
	ERRButton(string s, float x, float y, float wi, float h) 
		: Button(0, s, x,y,wi,h) {}
	
	virtual void click() {showErrors=!showErrors;}

	virtual void draw() {
		if (showErrors) setBgColor(128,128,128);
		else setBgColor(220,220,220);
		Button::draw();
	}
};

struct RegButton : Button {
	RegButton(GuiElement* p=0, string t="", float ix=0, float iy=0, float w=10, float h=10) 
		: Button(p,t, ix,iy,w,h) {setBgColor(180,180,180);}
	
	virtual void draw() {
		GuiElement::draw();
		glBegin(GL_LINE_LOOP);			
			glColor3f(.25f,.25f,.25f);
			glVertex2f(0,  0);
			glVertex2f(0, height);
			glVertex2f(width, height);
			glVertex2f(width, 0);
		glEnd();
		glColor3f(0,0,0);
		glEnable(GL_TEXTURE_2D);
		Font::Print(text, .5f, .5f, 2);
		glDisable(GL_TEXTURE_2D);
	}
};

struct BInstButton : RegButton {
	BInstButton(GuiElement* p, float x, float y, float wi, float h) 
		: RegButton(p, "Build Instances", x,y,wi,h) {}

	virtual void click() {Operation::instanceOperations();}
};

struct DupButton : RegButton {
	DupButton(GuiElement* p, float x, float y, float wi, float h) 
		: RegButton(p, "No Duplicates", x,y,wi,h) {}

	virtual void click() {Operation::noDups=!Operation::noDups;}

	virtual void draw() {
		if (Operation::noDups) setBgColor(128,128,128);
		else setBgColor(220,220,220);
		RegButton::draw();
	}
};

struct CnstButton : RegButton {
	CnstButton(GuiElement* p, float x, float y, float wi, float h) 
		: RegButton(p, "Use Constraints", x,y,wi,h) {}

	virtual void click() {Operation::ignoreCons=!Operation::ignoreCons;}

	virtual void draw() {
		if (!Operation::ignoreCons) setBgColor(128,128,128);
		else setBgColor(220,220,220);
		RegButton::draw();
	}
};

struct RedButton : RegButton {
	RedButton(GuiElement* p, float x, float y, float wi, float h) 
		: RegButton(p, "Filter Redundant", x,y,wi,h) {}

	virtual void click() {Operation::filterRedOps=!Operation::filterRedOps;}

	virtual void draw() {
		if (Operation::filterRedOps) setBgColor(128,128,128);
		else setBgColor(220,220,220);
		RegButton::draw();
	}
};

#endif