#include "ogl.h"

#include "ocbutton.h"
#include "ifwindow.h"
#include "pgwindow.h"
#include "dmwindow.h"
#include "logwindow.h"
#include "instwindow.h"

#include "../parser.h"
#include "../solver.h"
#include "../graphplan.h"
#include <stdio.h>
#include <string>
#include <vector>
#include <deque>
using std::deque;
using std::pair;

deque<std::string> history;
std::string input_string;
size_t cursor_pos=0;
int hist_pos=-1;

bool showErrors=1;

extern std::vector<token> tokenize(const char* s);
extern void parse(std::vector<token> tok);

static int mouseBtnState[10];

void mouseBtnCallback(int btn, int state) {
	mouseBtnState[btn]=state;
	if (btn==0 && state) guiRoot.mouseOver->click();
}

void mouseMoveCallback(int x, int y) {
	float oldX = guiRoot.mousePos[0]*100.0f/guiRoot.WinX;
	float oldY = guiRoot.mousePos[1]*100.0f/guiRoot.WinY;
	float newX = x*100.0f/guiRoot.WinX;
	float newY = y*100.0f/guiRoot.WinY;
	GuiElement* lastOver = guiRoot.mouseOver;
	guiRoot.mouseOver = guiRoot.pickElement(newX,newY);
	if (!guiRoot.mouseOver) guiRoot.mouseOver = &guiRoot;
	if (guiRoot.mouseOver!=lastOver) {
		lastOver->mouseLeave();
		guiRoot.mouseOver->mouseEnter();
	}
	else {
		if (mouseBtnState[0]) guiRoot.mouseOver->mouseDrag(newX-oldX, newY-oldY);
		else if (mouseBtnState[1]) guiRoot.mouseOver->wheel(newY-oldY);
		else guiRoot.mouseOver->mouseMove(newX, newY);
	}
	guiRoot.mousePos[0]=x; guiRoot.mousePos[1]=y;
}

//Wild offcenter zoom to/from mouse position (just feels better)
void mouseWheelCallback(int pos) {
	guiRoot.mouseOver->wheel((float)pos-guiRoot.mousePos[2]);
	guiRoot.mousePos[2]=pos;
}

void resizeCallback(int width, int height) {
	glfwSetWindowSize(width,height);
	glViewport(0,0,width,height);
	guiRoot.WinX=width; guiRoot.WinY=height;
}

void keyCallback(int key, int state) {
	if (!state) return;

	if (key==GLFW_KEY_ENTER || key==GLFW_KEY_KP_ENTER) {
		if (input_string.empty()) return;
		error.clear();
		parse(tokenize(input_string.c_str()));
		history.push_front(input_string);
		if (history.size()>20) history.pop_back();
		input_string.clear();
		cursor_pos=0;
		hist_pos=-1;
	}
	else if (key==GLFW_KEY_BACKSPACE) {
		if (cursor_pos>0) input_string.erase(--cursor_pos,1);
	}
	else if (key==GLFW_KEY_DEL) {
		if (cursor_pos<input_string.size()) input_string.erase(cursor_pos,1);
	}
	else if (key==GLFW_KEY_LEFT) {
		if (cursor_pos>0) --cursor_pos;
	}
	else if (key==GLFW_KEY_RIGHT) {
		if (++cursor_pos>input_string.size()) cursor_pos=input_string.size();
	}
	else if (key==GLFW_KEY_UP) {
		if (hist_pos+1<(int)history.size()) {
			input_string=history[++hist_pos];
			cursor_pos=input_string.size();
		}
	}
	else if (key==GLFW_KEY_DOWN) {
		if (hist_pos>0) {
			input_string=history[--hist_pos];
			cursor_pos=input_string.size();
		}
	}
}

void charCallback(int key, int state) {
	if (state) {
		input_string.insert(cursor_pos, string()+(char)key);
		cursor_pos++;
	}
}

void init_ogl() {
	if (!glfwInit()) {
		printf("GLFW failed to initialize!\n");
		return;
	}
	if(!glfwOpenWindow(guiRoot.WinX,guiRoot.WinY, 8,8,8,8, 16,0,GLFW_WINDOW)) {
		printf("Could not set Video Mode\n");
		glfwTerminate(); 
		return;
	}
	glfwSetWindowTitle("Plankram");
	glfwGetMousePos(&guiRoot.mousePos[0], &guiRoot.mousePos[1]);
	guiRoot.mousePos[2]=glfwGetMouseWheel();
	glfwSetWindowSizeCallback(resizeCallback);
	glfwSetMousePosCallback(mouseMoveCallback);
	glfwSetMouseButtonCallback(mouseBtnCallback);
	glfwSetMouseWheelCallback(mouseWheelCallback);
	glfwSetKeyCallback(keyCallback);
	glfwSetCharCallback(charCallback);
	glfwEnable(GLFW_KEY_REPEAT);

	glMatrixMode(GL_PROJECTION);
	float PMtr[16]={.02f,0,0,0,	 0,-.02f,0,0,  0,0,-1,0,  -1,1,0,1};
	glLoadMatrixf(PMtr);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	glClearColor(.7f,.7f,.7f,1.0f);
	Font::Init("verdanasmall.glf");

	guiRoot.setBgColor(200,200,200);
	DMWindow* dmWindow=new DMWindow;
	OIWindow* instWindow=new OIWindow;
	IFWindow* ifTable=new IFWindow;
	PGWindow* pgWindow=new PGWindow;
	LogWindow* logWindow=new LogWindow;
	new OCButton(*dmWindow, "Domain", 0,.0f,2.5f,15.0f);
	new OCButton(*instWindow, "Instances", 0,15.0f,2.5f,21.0f);
	new OCButton(*pgWindow, "Plangraph", 0,36.0f,2.5f,21.0f);
	new OCButton(*ifTable, "Interference", 0,57.0f,2.5f,27.0f);
	new OCButton(*logWindow, "Log", 0,84.0f,2.5f,10.0f);
	new ERRButton("^^", 0, 94.0f,2.5f,6.1f);
}

void disp_ogl() {
	if (!glfwGetWindowParam(GLFW_OPENED)) exit(0);
	glfwSwapBuffers();
	glClear(GL_DEPTH_BUFFER_BIT);

	static double lastTime= glfwGetTime();
	double time=glfwGetTime();
	float dtime=(float)(time-lastTime);
	lastTime=time;

	guiRoot.update(dtime);

	glDisable(GL_TEXTURE_2D);
	guiRoot.render();
	if (guiRoot.popUp) {
		glPushMatrix();
		glTranslatef(100.0f*(float)guiRoot.mousePos[0]/guiRoot.WinX, 
			4+100.0f*(float)guiRoot.mousePos[1]/guiRoot.WinY, 0);
		guiRoot.popUp->render();
		glPopMatrix();
	}

	float top = 95.5f;
	if (showErrors) top-=error.logs.size()*2.5f;
	glColor3f(.75f, .75f, .75f);
	glBegin(GL_QUADS);
		glVertex2f(2.65f, top);
		glVertex2f(2.65f, 100);
		glVertex2f(100, 100);
		glVertex2f(100,  top);
	glEnd();
	glColor3f(.35f,.35f,.35f);
	glBegin(GL_LINES);
		glVertex2f(2.65f, top);
		glVertex2f(100, top);
		glVertex2f(2.65f, top);
		glVertex2f(2.65f, 100);
	glEnd();

	glEnable(GL_TEXTURE_2D);	
	glColor3f(0,0,0);
	top-=2.0f;
	if (showErrors) {
		for (vector<string>::iterator it=error.logs.begin(); it!=error.logs.end(); it++)
			Font::Print(*it, 4, top+=2.5f,2);
	}
	Font::Print(string("> ")+input_string.substr(0,cursor_pos)+"_"+input_string.substr(cursor_pos), 4,97.0f,2);	
}
