#ifndef GUI_PGWINDOW_INCLUDED
#define GUI_PGWINDOW_INCLUDED

#include "window.h"
#include "../graphplan.h"
#include <cmath>


using std::pair;

static inline void drawBox(pair<float,float>& coord, float w) {
	glBegin(GL_LINE_LOOP);
	glVertex2f(coord.first-w, coord.second-1);
	glVertex2f(coord.first-w, coord.second+3);
	glVertex2f(coord.first+w, coord.second+3);
	glVertex2f(coord.first+w, coord.second-1);
	glEnd();
}

struct PGWindow : Window {
	PlanGraph& g;
	float scaleFac;
	float posOffset[2];
	PGWindow() 
		: Window(&guiRoot,"Plangraph",2.65f,.1f, 97.25f, 99.8f), g(graphSolver.graph), scaleFac(1.0f) {
			posOffset[0]=-50; posOffset[1]=-50;
			setBgColor(128,180,255);
	}

	virtual void wheel(float dz) {		
		float scaleOld = scaleFac;
		scaleFac*=1+dz*.1f;
		posOffset[0]+=(mousePos[0]*100.0f - 50*WinX)*(scaleOld - scaleFac) / (WinX*scaleOld*scaleFac);
		posOffset[1]+=(mousePos[1]*100.0f - 50*WinY)*(scaleOld - scaleFac) / (WinY*scaleOld*scaleFac);
	}
	virtual void mouseDrag(float dx, float dy) {
		posOffset[0]+=dx/scaleFac;
		posOffset[1]+=dy/scaleFac;
	}

	virtual void draw() {
	if (!isOpen && !isOpening) return;
	Window::draw();
	if (g.predLayers.empty()) {close(); return;}

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	float sz=50.0f/scaleFac;
	glOrtho(-sz, sz, sz, -sz, -1,1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(posOffset[0], posOffset[1], 0);

	float mpx=((guiRoot.mousePos[0]*100.0f/guiRoot.WinX)-50)/scaleFac - (x+posOffset[0]);
	float mpy=((guiRoot.mousePos[1]*100.0f/guiRoot.WinY)-50)/scaleFac - (y+posOffset[1]);

	size_t nodeUnderMouse=0, rowUnderMouse=0, typeUnderMouse=0;
	glColor3f(0,0,0);
	glEnable(GL_TEXTURE_2D);
	map<pair<size_t,size_t>, pair<float, float> > opCoords;
		map<pair<size_t,size_t>, pair<float, float> > predCoords;
		for (size_t i=0; i<g.predLayers.size()-1; i++) {
			float c=0;			
			for (size_t p = 0; p<g.predLayers[i+1]; p++) {
				float w = Font::Print(g.predNodes[p].ptr->toString(), c, (float)i*50, 2);
				if (mpy>(float)i*50-1 && mpy<(i*50)+3 && mpx>c-1.0f && mpx<c+w+1.0f) {
					nodeUnderMouse=p;
					rowUnderMouse=i;
					typeUnderMouse=1;
				}
				predCoords[pair<size_t,size_t>(p,i)]=pair<float, float>(c+w/2.0f,(float)i*50);
				c+=w+2.0f;
			}
		}
		for (size_t i=1; i<g.opLayers.size()-1; i++) {
			float c=0;
			for (size_t o = 0; o<g.opLayers[i+1]; o++) {
				if (g.opNodes[o].ptr) continue;
				c=predCoords[pair<size_t,size_t>(g.opNodes[o].pre.front(),i-1)].first;
				if (mpy>(float)i*50-26 && mpy<(float)i*50-22 && mpx>c-2.0f && mpx<c+2.0f) {
					nodeUnderMouse=o;
					rowUnderMouse=i;
					typeUnderMouse=2;
				}
				opCoords[pair<size_t,size_t>(o,i)]=pair<float, float>(c,(float)i*50-25);
			}
			c+=4.0f;
			for (size_t o = 0; o<g.opLayers[i+1]; o++) {
				if (!g.opNodes[o].ptr) continue;
				float w = Font::Print(g.opNodes[o].ptr->toString(), c, (float)i*50-25, 2);
				if (mpy>(float)i*50-26 && mpy<(i*50)-22 && mpx>c-1.0f && mpx<c+w+1.0f) {
					nodeUnderMouse=o;
					rowUnderMouse=i;
					typeUnderMouse=2;
				}
				opCoords[pair<size_t,size_t>(o,i)]=pair<float, float>(c+w/2.0f,(float)i*50-25);
				c+=w+2.0f;
			}
		}

		glDisable(GL_TEXTURE_2D);
		glBegin(GL_LINES);
		for (size_t i=1; i<g.opLayers.size()-1; i++) {
			for (size_t o=0; o<g.opLayers[i+1]; o++) {
				glColor3f(0,1,0);
				for (vector<size_t>::const_iterator l=g.opNodes[o].add.begin(); l!=g.opNodes[o].add.end(); l++) {
					glVertex2f(	opCoords[pair<size_t,size_t>(o,i)].first, 
								opCoords[pair<size_t,size_t>(o,i)].second+3.0f);
					glVertex2f(	predCoords[pair<size_t,size_t>(*l,i)].first, 
								predCoords[pair<size_t,size_t>(*l,i)].second-1.0f);
				}
				glColor3f(0,0,1);
				for (vector<size_t>::const_iterator l=g.opNodes[o].pre.begin(); l!=g.opNodes[o].pre.end(); l++) {
					glVertex2f(	predCoords[pair<size_t,size_t>(*l,i-1)].first, 
								predCoords[pair<size_t,size_t>(*l,i-1)].second+3.0f);
					glVertex2f(	opCoords[pair<size_t,size_t>(o,i)].first, 
								opCoords[pair<size_t,size_t>(o,i)].second-1.0f);
				}
			}
		}
		glEnd();

		if (typeUnderMouse>0) {
			glLineWidth(2);
			float w=0;
			pair<float,float> coord;
			
			if (typeUnderMouse==1) {
				glColor3f(1,1,0);
				coord = predCoords[pair<size_t,size_t>(nodeUnderMouse,rowUnderMouse)];
				w=Font::width(g.predNodes[nodeUnderMouse].ptr->toString(), 2)/2.0f+.75f;
				drawBox(coord,w);
				glColor3f(1,0,0);
				for (size_t n=0; n<g.predLayers[rowUnderMouse+1]; n++) {
					if (g.predNodes[nodeUnderMouse].excl[rowUnderMouse].get(n)) {
						coord = predCoords[pair<size_t,size_t>(n,rowUnderMouse)];
						w=Font::width(g.predNodes[n].ptr->toString(), 2)/2.0f+.75f;
						drawBox(coord,w);
					}
				}
			}
			if (typeUnderMouse==2) {
				glColor3f(1,1,0);
				coord = opCoords[pair<size_t,size_t>(nodeUnderMouse,rowUnderMouse)];
				float w=2;
				if (g.opNodes[nodeUnderMouse].ptr) 
					w=Font::width(g.opNodes[nodeUnderMouse].ptr->toString(), 2)/2.0f+.75f;
				drawBox(coord,w);
				glColor3f(1,0,0);
				for (size_t n=0; n<g.opLayers[rowUnderMouse+1]; n++) {
					if (g.opNodes[nodeUnderMouse].excl[rowUnderMouse].get(n)) {
						coord = opCoords[pair<size_t,size_t>(n,rowUnderMouse)];
						float w=2;
						if (g.opNodes[n].ptr) w=Font::width(g.opNodes[n].ptr->toString(), 2)/2.0f+.75f;
						drawBox(coord,w);
					}
				}
			}
			glLineWidth(1);
		}

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	}
};

#endif