#include "parser.h"
#include "operation.h"
#include "display.h"
#include "solver.h"
#include <string>
#include <vector>
using std::string;
using std::vector;

bool consoleNeedsUpdate=1;
//You don't want to see this file. Trust me. You don't. Not even relevant to the actual task.
//Plus, calling this "parser" is a ridiculous overstatement

Constraint* getConst(Operation* op, vector<token>& tok, int& i, bool neg) {
	Variable* a = op->getVariable(tok[++i].txt.c_str());
	if (!a) {
		error("A variable is required as first operand\n");
		return 0;
	}
	ConstInOf* c = Constraint::InOf(a, neg);
	for (++i; i<(int)tok.size(); i++) {
		if (tok[i].type!=token::PARAM) {--i; break;}
		Type* t = Type::getByName(tok[i].txt.c_str());
		if (t) c->T.insert(t);
		else {
			Atom* a = op->getVariable(tok[i].txt.c_str());
			if (!a) a = Atom::getByName(tok[i].txt.c_str());
			if (a) c->A.insert(a);
			else error(tok[i].txt+" not found\n");
		}
	}
	if (!c->A.empty() || !c->T.empty()) return c;
	delete c;
	return 0;
}

void readConstraint(vector<token>& tok) {
	Operation* op = Operation::getByName(tok[1].txt.c_str());
	if (!op) {error("No operation \"" + tok[1].txt + "\"\n"); return;}

	CondConst* cc=0;
	set<Constraint*>* sp=&op->constraints;	
	int first=2;

	if (tok[0].txt=="condconst") {
		cc = new CondConst();
		sp = &cc->conditions;
		if (tok[first].txt=="all") {++first; cc->bOr=0;}
		else if (tok[first].txt=="one") {++first; cc->bOr=1;}
	}
	for (int i=first; i<(int)tok.size(); i++) {
		if (cc && tok[i].txt=="=>") sp=&cc->consts;
		if (tok[i].txt=="eq" || tok[i].txt=="in" ||	tok[i].txt=="is") {
			Constraint *c=getConst(op, tok, i, tok[i-1].type==token::NEG);
			if (c) sp->insert(c);
		}
	}
	if (cc && !cc->conditions.empty() && !cc->consts.empty()) op->addConstraint(cc);
}


//Wildly iterating over all read tokens and turning them into predicates with parameters
void setState(vector<token>& tok, State& s, Operation* op=0) {
	s.preds.clear();
	int i= op ? 2 : 1, numArgs=(int)tok.size();
	while (i<numArgs) {
		if (tok[i].type==token::WORD) {
			string& name=tok[i].txt;
			vector<Atom*> par;
			while (++i<numArgs && tok[i].type==token::PARAM) {
				Atom* a = Atom::getByName(tok[i].txt.c_str());
				if (!a && op) a = op->getVariable(tok[i].txt.c_str());
				if (a) par.push_back(a);
			}
			s.add(Predicate::instance(name.c_str(), par));
		}
		else i++;
	}
}

vector<token> tokenize(const char* s) {
	const char* p = s;
	bool inWord=0, inParam=0;
	vector<token> result;

	while (*p) {
		if (*p==' ' || *p==',' || *p=='^' || *p=='(' || *p==')' || *p=='\n' || *p==':' || *p=='|' || *p=='=') {
			if (inWord) inWord=0;
			if (*p=='(') inParam=1;
			if (*p==')') inParam=0;
		}
		else if (*p=='>' && p>s && *(p-1)=='=') {
			token t={"=>",token::WORD};
			result.push_back(t);
		}
		else if (*p=='~') {
			token t={"~",token::NEG};
			result.push_back(t);
		}
		else {			
			if (!inWord) {
				inWord=1;
				result.push_back(token());
				result.back().type = inParam ? token::PARAM : token::WORD;
			}
			if (inWord) result.back().txt+=*p;
		}		
		p++;
	}
	return result;
}

void parse(vector<token> tok) {
	consoleNeedsUpdate=1;
	int numArgs = (int)tok.size();
	if (!numArgs || tok[0].txt[0]=='#') return;

	//Somebody must have had enough
	if (tok[0].txt=="q" || tok[0].txt=="quit" || tok[0].txt=="exit") exit(0);
	
	//Load stuff from file, because I'm just too damn lazy
	else if (tok[0].txt=="file" && numArgs>1) {
		if (tok[1].txt.size()<5 || tok[1].txt.substr(tok[1].txt.size()-5, 4)!=".pln") tok[1].txt+=".pln";
		loadFromFile(tok[1].txt.c_str());
	}

	//Toggle logging (because it completely kills the performance)
	else if (tok[0].txt=="log") logger.enabled=!logger.enabled;

	//Init- and Goal-State
	else if(tok[0].txt=="init") setState(tok, init);
	else if(tok[0].txt=="goal") setState(tok, goal);

	//Pre/Add/Del for last operation set
	else if (tok[0].txt=="pre" && numArgs>2) {
		Operation* op = Operation::getByName(tok[1].txt.c_str());
		if (op) setState(tok, op->pre, op);
		else error("No operation \"" + tok[1].txt + "\"\n");
	}
	else if (tok[0].txt=="add" && numArgs>2) {
		Operation* op = Operation::getByName(tok[1].txt.c_str());
		if (op) setState(tok, op->add, op);
		else error("No operation \"" + tok[1].txt + "\"\n");
	}
	else if (tok[0].txt=="del" && numArgs>2) {
		Operation* op = Operation::getByName(tok[1].txt.c_str());
		if (op) setState(tok, op->del, op);
		else error("No operation \"" + tok[1].txt + "\"\n");
	}

	else if (numArgs>4 && (tok[0].txt=="condconst" || tok[0].txt=="const")) readConstraint(tok);

	//New operation entered
	else if (tok[0].txt=="op" && numArgs>1) {
		Operation* op = Operation::create(tok[1].txt.c_str());
		if (!op->params.empty()) return; //Obviously already existed
		for (int i=2; i+1<numArgs; i++) {
			Type* t = Type::getByName(tok[i].txt.c_str());
			if (t) op->addVariable(Variable::create(tok[++i].txt.c_str(), t));
			else error("Type \"" + tok[i].txt + "\" unknown\n");
		}
	}

	//New predicate entered
	else if (tok[0].txt=="pred" && numArgs>1) {
		Predicate* p = Predicate::create(tok[1].txt.c_str());
		if (!p->params.empty()) return; //Obviously already existed
		for (int i=2; i+1<numArgs; i++) {
			Type* t = Type::getByName(tok[i].txt.c_str());
			if (t) p->addVariable(Variable::create(tok[++i].txt.c_str(), t));
			else error("Type \"" + tok[i].txt + "\" unknown\n");
		}
	}

	//Toggle display of operation instances (handy for debugging)
	else if (tok[0].txt=="showInst") showInstances=!showInstances;

	//Create operation instance for testing
	else if (tok[0].txt=="instance" && numArgs>2) {
		vector<Atom*> par;
		for (int i=2; i<numArgs; i++) {
			Atom* a = Atom::getByName(tok[i].txt.c_str());
			if (a) par.push_back(a);
			else error("\""+tok[i].txt+"\" not found or no atom\n");
		}
		Operation::instance(tok[1].txt.c_str(), par);
	}

	else if (tok[0].txt=="buildInst") {
		if (Solver::solution) Solver::solution->clear(); //Important, don't want dangling pointers in old plans
		Operation::instanceOperations();
	}

	else if (tok[0].txt=="wipe") Solver::cleanUp();

	else if (tok[0].txt=="solve" && numArgs>1) {
		if (numArgs>2) Solver::runSolver(tok[1].txt, atoi(tok[2].txt.c_str()));
		else Solver::runSolver(tok[1].txt);
	}

	else if (tok[0].txt=="noDups") Operation::noDups=!Operation::noDups;
	else if (tok[0].txt=="ignConst") Operation::ignoreCons=!Operation::ignoreCons;
	else if (tok[0].txt=="filterReds") Operation::filterRedOps=!Operation::filterRedOps;

	//New type(s) entered
	else if(tok[0].txt=="type") {
		for (int i=1; i<numArgs; i++) Type::create(tok[i].txt.c_str());
	}

	//New derived type
	else if (tok[0].txt=="derived" && numArgs>2) {
		Type* t = Type::getByName(tok[1].txt.c_str());
		if (t) 
			for (int i=2; i<numArgs; i++) Type::create(tok[i].txt.c_str(), t);
		else error("No operation \"" + tok[1].txt + "\"\n");
	}

	//New atom(s) entered
	else if(tok[0].txt=="atom" && numArgs>2) {
		Type* t = Type::getByName(tok[1].txt.c_str());
		if (t!=anyType) {
			if (t) for (int i=2; i<numArgs; i++) Atom::create(tok[i].txt.c_str(), t);
			else error("Type \"" + tok[1].txt + "\" unknown\n");
		}
	}

	else error("Unknown command or not enough parameters\n");
}

void loadFromFile(const char* name) {
	FILE* file=fopen(name,"r");
	if (!file) {
		error(string("File \"") + name + "\" not found\n");
		return;
	}

	Solver::cleanUp();
	char tmp[256];
	while (fgets(tmp, 255, file)) parse(tokenize(tmp));
	fclose(file);
}


void input() {
	char tmp[256];
	fgets(tmp, 255, stdin);
	error.clear();
	parse(tokenize(tmp));
}
