#include "predicate.h"

list<Predicate*> Predicate::instances;

void Predicate::wipe() {
	deepWipe();
	for (list<Predicate*>::iterator it=instances.begin(); it!=instances.end(); it++) delete *it;
	instances.clear();
}

string Predicate::toString() const {
	string s=name+"(";
	for (vector<Atom*>::const_iterator it=params.begin(); it!=params.end(); it++) {
		if (it!=params.begin()) s+= isProto ? ", " : ",";
		if (isProto) s+=(*it)->type->name+":";
		s+=(*it)->name;
		
	}
	return s+=")";
}

void Predicate::print() const {printf("%s", toString().c_str()) ;}

Predicate* Predicate::instance(const char* n, vector<Atom*>& param) {
	//Search prototype for Predicate
	Predicate* p = getByName(n);

	//Doublecheck if prototype was found and number of arguments matches
	if (!p) {
		error(string("Predicate \"")+n+"\" doesn't exist\n");
		return 0;
	}
	if (param.size()!=p->params.size()) {
		error("Wrong number of parameters\n");
		return 0;
	}
	//Make sure all passed parameters have the correct type
	for (int i=0; i<(int)param.size(); i++) {
		if (param[i]->type!=p->params[i]->type && p->params[i]->type!=anyType) {
			error("Type mismatch, "+param[i]->name+" is type "+param[i]->type->name+
						", but "+p->params[i]->type->name+" expected\n");
			return 0;
		}
	}

	//Avoid duplicates
	for (list<Predicate*>::iterator it=instances.begin(); it!=instances.end(); it++)
		if ((*it)->name==n && (*it)->params==param) return *it;
	
	//Create and return predicate using passed parameters
	Predicate* np = new Predicate(n);
	np->params=param;
	np->isProto=0;
	instances.push_back(np);
	return np;
}
