#include "solver.h"

map<string, Solver*>* Solver::solvers = 0;
Plan* Solver::solution=0;

//We will always need them anyway, so they are global. Sue me.
State init, goal;
Plan plan;

void Solver::cleanUp() {
	Type::wipe();
	Atom::wipe();
	Variable::wipe();
	Predicate::wipe();
	Operation::wipe();
	init.preds.clear();
	goal.preds.clear();
	if (solution) solution->clear();
	solution=0;
	logger.clear();

	for (map<string, Solver*>::iterator it = solvers->begin(); it!=solvers->end(); it++)
		it->second->clear();
}

void Solver::runSolver(string& s, int p) {
	map<string, Solver*>::iterator it = solvers->find(s);
	if (it==solvers->end()) {error("Unknown solver\n"); return;}

	Solver* solver = it->second;
	if (solution) solution->clear();
	solution=0;
	logger.clear();

	uint64 time=ticks();
	bool solved = solver->solve(init, goal, p);
	time=ticks()-time;

	logger(string("\nSolution ") + (solved ? "found":"not found") + "\n");
	if (!solved) plan.steps.clear(); //Just in case
	char tmp[128];
	float dt = (float)time/2000000.0f;
	unsigned t = (unsigned)(time/1000);
	_snprintf(tmp, 128, " - required %uk clock cycles (%.4fms at 2Ghz)\n", t, dt);
	error(tmp);
}
