#ifndef SOLVER_INCLUDED
#define SOLVER_INCLUDED

#include "plan.h"

extern State init, goal;

//Who wants to type THAT all the time?
typedef unsigned long long uint64;

//Replace ASM with correct version for your compiler
//Note, this test is depending on the project file for VC++
//TODO: replace with correct compiler defines
//Great, GCC can't handle inlines from source files
#ifdef _CRT_SECURE_NO_DEPRECATE
inline volatile uint64 ticks() {
	unsigned t_high, t_low;
	__asm {
		CPUID
		RDTSC
		mov	t_low, eax
		mov t_high, edx
	}
	return ((uint64)t_high<<32)|t_low;
}
#else
inline volatile uint64 ticks() {
   register uint64 time asm("eax");
   asm volatile ("cpuid");
   asm volatile ("rdtsc" : : : "eax", "edx");
   return time;
}
#endif


//Radical change: I'm too lazy to update 4 files, everytime I add a new solver, so from now
//on all solvers are derived from this and enter themselves into the map in their contructor
//Result: no more updating of solver.h, parser.cpp and solver.cpp necessary
class Solver {
	static::map<string, Solver*>* solvers;

public:
	static Plan* solution;

	Solver(const char* n) {
		if (!solvers) solvers = new map<string, Solver*>;
		(*solvers)[n]=this;
	}

	virtual bool solve(const State& s, const State& g, int p=0) = 0;
	
	static void runSolver(string& s, int p=0);
	static void cleanUp();

	virtual void clear() {}
	//Check if all predicates of goal state are part of the tested state
	inline bool goalReached(const State& s, const State& g) {
		for (set<Predicate*>::const_iterator it=g.preds.begin(); it!=g.preds.end(); it++) {
			if (!s.elem(*it)) return 0;
		}
		return 1;
	}

	//Almost the same, but counting the actual number of open goals
	inline int openGoals(const State& s, const State& g) {
		int num=0;
		for (set<Predicate*>::const_iterator it=g.preds.begin(); it!=g.preds.end(); it++)
			if (!s.elem(*it)) num++;
		return num;
	}
};

#endif
