#include "state.h"
#include "plan.h"

//Literally: no comment
void State::add(Predicate* const p) {if (p) preds.insert(p);}

//For whatever reason there is no remove function
//Maybe one is hiding in the algorithm-header, but so what
void State::del(Predicate* const p) {
	if (p) {
		set<Predicate*>::iterator it=preds.find(p);
		if (it!=preds.end()) preds.erase(it);
	}
}

//Straightforward: Is p part of this state or not?
//Problem: We fucked up the set by smuggling duplicates into it, which breaks the find-method
//Solution: This is called a lot, so we (meaning I) prefer extra work when instancing operators
//over increasing the time for this test by making it linear (and still have a broken set)
bool State::elem(Predicate* const p) const {return preds.find(p)!=preds.end();}

bool State::operator<(const State& s) const {return preds<s.preds;}
//Window dressing: hopefully nobody notices that this whole ^ is quite pointless. Oops, too late
void State::print() const {printf("%s", toString().c_str());}

string State::toString() const {
	string s;
	for (set<Predicate*>::const_iterator it=preds.begin(); it!=preds.end(); it++) {
		if (it!=preds.begin()) s+=", ";
		s+=(*it)->toString();
	}
	return s+="\n";
}

State& State::operator+=(const Plan& p) {
	list<Operation*>::const_iterator it=p.steps.begin();
	for (; it!=p.steps.end(); it++) *this+=**it;
	return *this;
}

//Apply operation to state if possible, checking, deleting, adding all Predicates
//And yes, order matters if you pull stunts like adding Clear(Floor)
//Isn't that against SOME rule of STRIPS anyway?
State& State::operator+=(const Operation& op) {	
	set<Predicate*>::const_iterator it;	
	//Check preconditions
	for (it=op.pre.preds.begin(); it!=op.pre.preds.end(); it++) if (!elem(*it))
		return *this;

	//Apply effects
	for (it=op.del.preds.begin(); it!=op.del.preds.end(); it++) del(*it);
	for (it=op.add.preds.begin(); it!=op.add.preds.end(); it++) add(*it);
	return *this;
}
